@@ -1,44 +1,32 @@
 from __future__ import annotations
 
-import functools
-import typing
 from collections import deque
 from contextvars import ContextVar
-from dataclasses import dataclass
 from typing import (
     Any,
     ContextManager,
     Deque,
     Dict,
-    FrozenSet,
     Iterable,
     List,
     Mapping,
-    NamedTuple,
     Optional,
-    Set,
-    Tuple,
     Union,
     cast,
 )
 
 from di._dag import topsort
+from di._execution_planning import SolvedDependencyCache, plan_execution
 from di._inspect import is_async_gen_callable, is_coroutine_callable
 from di._local_scope_context import LocalScopeContext
 from di._nullcontext import nullcontext
 from di._state import ContainerState
-from di._task import AsyncTask, ExecutionState, SyncTask, Task
-from di.exceptions import (
-    DependencyRegistryError,
-    DuplicateScopeError,
-    ScopeViolationError,
-    UnknownScopeError,
-)
+from di._task import AsyncTask, SyncTask, Task
+from di.exceptions import DependencyRegistryError, DuplicateScopeError
 from di.executors import DefaultExecutor
 from di.types import FusedContextManager
 from di.types.dependencies import DependantProtocol, DependencyParameter
 from di.types.executor import AsyncExecutor, SyncExecutor
-from di.types.executor import Task as ExecutorTask
 from di.types.providers import (
     DependencyProvider,
     DependencyProviderType,
@@ -50,20 +38,6 @@ from di.types.solved import SolvedDependency
 Dependency = Any
 
 
-class _ExecutionPlanCache(NamedTuple):
-    cache_key: FrozenSet[DependantProtocol[Any]]
-    dependant_dag: Mapping[DependantProtocol[Any], Iterable[Task[Any]]]
-    dependency_counts: Dict[DependantProtocol[Any], int]
-    leaf_tasks: Iterable[Task[Any]]
-
-
-@dataclass
-class _SolvedDependencyCache:
-    tasks: Mapping[DependantProtocol[Any], Task[Any]]
-    dependency_dag: Mapping[DependantProtocol[Any], Set[DependantProtocol[Any]]]
-    execution_plan: Optional[_ExecutionPlanCache] = None
-
-
 class Container:
     _context: ContextVar[ContainerState]
     _executor: Union[AsyncExecutor, SyncExecutor]
@@ -192,7 +166,7 @@ class Container:
                         q.append(subdep)
         tasks = self._build_tasks(param_graph, topsort(dep_dag))
         dependency_dag = {dep: set(subdeps) for dep, subdeps in dep_dag.items()}
-        container_cache = _SolvedDependencyCache(
+        container_cache = SolvedDependencyCache(
             dependency_dag=dependency_dag, tasks=tasks
         )
         return SolvedDependency(
@@ -238,152 +212,6 @@ class Container:
         else:
             return SyncTask(dependant=dependency, dependencies=task_dependencies)
 
-    def _validate_scopes(self, solved: SolvedDependency[Dependency]) -> None:
-        """Validate that dependencies all have a valid scope and
-        that dependencies only depend on outer scopes or their own scope.
-        """
-        scopes: Dict[Scope, int] = {
-            scope: idx
-            for idx, scope in enumerate(reversed(self._state.scopes + [None]))
-        }
-
-        def check_is_inner(
-            dep: DependantProtocol[Any], subdep: DependantProtocol[Any]
-        ) -> None:
-            if scopes[dep.scope] > scopes[subdep.scope]:
-                raise ScopeViolationError(
-                    f"{dep} cannot depend on {subdep} because {subdep}'s"
-                    f" scope ({subdep.scope}) is narrower than {dep}'s scope ({dep.scope})"
-                )
-
-        def check_scope(dep: DependantProtocol[Any]) -> None:
-            if dep.scope not in scopes:
-                raise UnknownScopeError(
-                    f"Dependency{dep} has an unknown scope {dep.scope}."
-                    f" Did you forget to enter the {dep.scope} scope?"
-                )
-
-        for dep, params in solved.dag.items():
-            check_scope(dep)
-            for param in params:
-                subdep = param.dependency
-                check_scope(subdep)
-                check_is_inner(dep, subdep)
-
-    def _prepare_execution(
-        self,
-        solved: SolvedDependency[Any],
-        *,
-        validate_scopes: bool = True,
-        values: Optional[Mapping[DependencyProvider, Any]] = None,
-    ) -> Tuple[
-        Dict[DependantProtocol[Any], Any],
-        List[ExecutorTask],
-        typing.Iterable[DependantProtocol[Any]],
-    ]:
-        user_values = values or {}
-        if validate_scopes:
-            self._validate_scopes(solved)
-
-        if type(solved.container_cache) is not _SolvedDependencyCache:
-            raise TypeError(
-                "This SolvedDependency was not created by this Container"
-            )  # pragma: no cover
-
-        solved_dependency_cache = solved.container_cache
-
-        cache: Dict[DependencyProvider, Any] = {}
-        for mapping in self._state.cached_values.mappings.values():
-            cache.update(mapping)
-        cache.update(user_values)
-
-        to_cache: typing.Deque[DependantProtocol[Any]] = deque()
-        execution_scope = self.scopes[-1]
-
-        results: Dict[DependantProtocol[Any], Any] = {}
-
-        def use_cache(dep: DependantProtocol[Any]) -> None:
-            call = dep.call
-            if call in cache:
-                if call in user_values:
-                    results[dep] = user_values[call]
-                    return
-                elif dep.share:
-                    results[dep] = cache[call]
-                    return
-            else:
-                if dep.share and dep.scope != execution_scope:
-                    to_cache.append(dep)
-            return
-
-        for dep in solved.dag:
-            use_cache(dep)
-
-        execution_plan_cache_key = frozenset(results.keys())
-
-        execution_plan = solved.container_cache.execution_plan
-
-        if (
-            execution_plan is None
-            or execution_plan.cache_key != execution_plan_cache_key
-        ):
-            # Build a DAG of Tasks that we actually need to execute
-            # this allows us to prune subtrees that will come
-            # from pre-computed values (values paramter or cache)
-            unvisited = deque([solved.dependency])
-            # Make DAG values a set to account for dependencies that depend on the the same
-            # sub dependency in more than one param (`def func(a: A, a_again: A)`)
-            dependency_counts: Dict[DependantProtocol[Any], int] = {}
-            dependant_dag: Dict[DependantProtocol[Any], Deque[Task[Any]]] = {
-                dep: deque() for dep in solved_dependency_cache.dependency_dag
-            }
-            while unvisited:
-                dep = unvisited.pop()
-                if dep in dependency_counts:
-                    continue
-                # task the dependency is cached or was provided by value
-                # we don't need to compute it or any of it's dependencies
-                if dep not in results:
-                    # otherwise, we add it to our DAG and visit it's children
-                    dependency_counts[dep] = 0
-                    for subdep in solved_dependency_cache.dependency_dag[dep]:
-                        if subdep not in results:
-                            dependant_dag[subdep].append(
-                                solved_dependency_cache.tasks[dep]
-                            )
-                            dependency_counts[dep] += 1
-                            if subdep not in dependency_counts:
-                                unvisited.append(subdep)
-
-            solved.container_cache.execution_plan = (
-                execution_plan
-            ) = _ExecutionPlanCache(
-                cache_key=execution_plan_cache_key,
-                dependant_dag=dependant_dag,
-                dependency_counts=dependency_counts,
-                leaf_tasks=[
-                    solved_dependency_cache.tasks[dep]
-                    for dep, count in dependency_counts.items()
-                    if count == 0
-                ],
-            )
-
-        state = ExecutionState(
-            container_state=self._state,
-            results=results,
-            dependency_counts=execution_plan.dependency_counts.copy(),
-            dependants=execution_plan.dependant_dag,
-        )
-
-        return (
-            results,
-            [
-                functools.partial(t.compute, state)  # type: ignore[arg-type]
-                for t in execution_plan.leaf_tasks
-            ],
-            to_cache,
-        )
-
     def _update_cache(
         self,
         results: Dict[DependantProtocol[Any], Any],
@@ -414,8 +242,8 @@ class Container:
         else:
             cm = self.enter_local_scope(self._execution_scope)
         with cm:
-            results, queue, to_cache = self._prepare_execution(
-                solved, validate_scopes=validate_scopes, values=values
+            results, queue, to_cache = plan_execution(
+                self._state, solved, validate_scopes=validate_scopes, values=values
             )
             if not hasattr(self._executor, "execute_sync"):
                 raise TypeError(
@@ -447,8 +275,8 @@ class Container:
         else:
             cm = self.enter_local_scope(self._execution_scope)
         async with cm:
-            results, queue, to_cache = self._prepare_execution(
-                solved, validate_scopes=validate_scopes, values=values
+            results, queue, to_cache = plan_execution(
+                self._state, solved, validate_scopes=validate_scopes, values=values
             )
             if not hasattr(self._executor, "execute_async"):
                 raise TypeError(
