@@ -1,7 +1,14 @@
 from __future__ import annotations
 
 import inspect
-from typing import Any, Dict, Iterable, List, Optional, overload
+import sys
+from itertools import chain
+from typing import Any, Dict, Iterable, List, Mapping, Optional, Type, overload
+
+if sys.version_info < (3, 8):
+    from typing_extensions import Protocol
+else:
+    from typing import Protocol
 
 from di._docstrings import join_docstring_from
 from di._inspect import get_parameters, infer_call_from_annotation
@@ -23,7 +30,11 @@ _VARIABLE_PARAMETER_KINDS = (
 )
 
 
-class Dependant(DependantProtocol[DependencyType]):
+class DependantBase(DependantProtocol[DependencyType]):
+    """Concrete base class for Dependants"""
+
+
+class Dependant(DependantBase[DependencyType]):
     wire: bool
     autowire: bool
 
@@ -36,6 +47,7 @@ class Dependant(DependantProtocol[DependencyType]):
         share: bool = True,
         wire: bool = True,
         autowire: bool = True,
+        overrides: Optional[Mapping[str, DependantProtocol[Any]]] = None,
     ) -> None:
         ...
 
@@ -48,6 +60,7 @@ class Dependant(DependantProtocol[DependencyType]):
         share: bool = True,
         wire: bool = True,
         autowire: bool = True,
+        overrides: Optional[Mapping[str, DependantProtocol[Any]]] = None,
     ) -> None:
         ...
 
@@ -60,6 +73,7 @@ class Dependant(DependantProtocol[DependencyType]):
         share: bool = True,
         wire: bool = True,
         autowire: bool = True,
+        overrides: Optional[Mapping[str, DependantProtocol[Any]]] = None,
     ) -> None:
         ...
 
@@ -72,6 +86,7 @@ class Dependant(DependantProtocol[DependencyType]):
         share: bool = True,
         wire: bool = True,
         autowire: bool = True,
+        overrides: Optional[Mapping[str, DependantProtocol[Any]]] = None,
     ) -> None:
         ...
 
@@ -83,6 +98,7 @@ class Dependant(DependantProtocol[DependencyType]):
         share: bool = True,
         wire: bool = True,
         autowire: bool = True,
+        overrides: Optional[Mapping[str, DependantProtocol[Any]]] = None,
     ) -> None:
         self.call = call
         self.scope = scope
@@ -92,6 +108,7 @@ class Dependant(DependantProtocol[DependencyType]):
         self.share = share
         self.autowire = autowire
         self.wire = wire
+        self.overrides = overrides or {}
 
     def __repr__(self) -> str:
         share = "" if self.share is False else ", share=True"
@@ -151,10 +168,13 @@ class Dependant(DependantProtocol[DependencyType]):
         ), "Container should have assigned call; this is a bug!"
         res: List[DependencyParameter[DependantProtocol[Any]]] = []
         for param in self.gather_parameters().values():
-            if param.kind in _VARIABLE_PARAMETER_KINDS and self.autowire:
+            sub_dependant: DependantProtocol[Any]
+            if param.name in self.overrides:
+                sub_dependant = self.overrides[param.name]
+            elif param.kind in _VARIABLE_PARAMETER_KINDS and self.autowire:
                 continue
-            if isinstance(param.default, Dependant):
-                sub_dependant: DependantProtocol[Any] = param.default
+            elif isinstance(param.default, Dependant):
+                sub_dependant = param.default
             elif param.default is param.empty and self.autowire:
                 sub_dependant = self.create_sub_dependant(param)
             else:
@@ -192,28 +212,89 @@ class Dependant(DependantProtocol[DependencyType]):
         )
 
 
-class JoinedDependant(Dependant[DependencyType]):
+class JoinedDependant(DependantBase[DependencyType]):
+    """A Dependant that aggregates other dependants without directly depending on them"""
+
+    _dependencies: Optional[List[DependencyParameter[Any]]]
+
     def __init__(
         self,
         dependant: DependantProtocol[DependencyType],
         *,
         siblings: Iterable[DependantProtocol[Any]],
-        scope: Scope = None,
-        share: bool = True,
-        autowire: bool = True,
     ) -> None:
         self.call = dependant.call
         self.dependant = dependant
         self.siblings = siblings
-        self.scope = scope
-        self.dependencies: Optional[
-            List[DependencyParameter[DependantProtocol[Any]]]
-        ] = None
-        self.share = share
-        self.autowire = autowire
+        self.scope = dependant.scope
+        self._dependencies = None
+        self.share = dependant.share
 
-    def gather_dependencies(self) -> List[DependencyParameter[DependantProtocol[Any]]]:
-        return [
-            *self.dependant.get_dependencies(),
-            *(DependencyParameter(dep, None) for dep in self.siblings),
-        ]
+    def get_dependencies(self) -> List[DependencyParameter[DependantProtocol[Any]]]:
+        if self._dependencies is None:
+            self._dependencies = list(
+                chain(
+                    self.dependant.get_dependencies(),
+                    (DependencyParameter(dep, None) for dep in self.siblings),
+                )
+            )
+        return self._dependencies
+
+    def __hash__(self) -> int:
+        return hash((self.dependant, *self.siblings))
+
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, JoinedDependant):
+            return False
+        return (self.dependant, *self.siblings) == (o.dependant, *o.siblings)
+
+    def register_parameter(
+        self, param: inspect.Parameter
+    ) -> DependantProtocol[DependencyType]:
+        return self.dependant.register_parameter(param)
+
+
+class UniqueDependant(Dependant[DependencyType]):
+    """A Dependant that can be cached/shared but is never substituted with another Dependant in the DAG"""
+
+    # By overriding __eq__, __hash__ gets set to None
+    # see https://docs.python.org/3/reference/datamodel.html#object.__hash__ a couple paragraphs down
+    def __hash__(self) -> int:
+        return super().__hash__()
+
+    def __eq__(self, o: object) -> bool:
+        return False
+
+
+class CallableClass(Protocol[T]):
+    __call__: DependencyProviderType[T]
+
+
+def CallableClassDependant(
+    call: Type[CallableClass[T]],
+    *,
+    cls_provider: Optional[DependencyProviderType[CallableClass[Any]]] = None,
+    instance_scope: Scope = None,
+    scope: Scope = None,
+    share: bool = True,
+    wire: bool = True,
+    autowire: bool = True,
+) -> Dependant[T]:
+    if not (inspect.isclass(call) and hasattr(call, "__call__")):
+        raise TypeError("call must be a callable class")
+    cls_provider = cls_provider or call
+    self = UniqueDependant(
+        cls_provider,
+        scope=instance_scope,
+        share=True,
+        wire=wire,
+        autowire=autowire,
+    )
+    return Dependant(
+        call=call.__call__,
+        scope=scope,
+        share=share,
+        wire=wire,
+        autowire=autowire,
+        overrides={"self": self},
+    )
