@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import sys
-from typing import Awaitable, Iterable, Optional, Union
+from typing import Awaitable, Iterable, Union
 
 if sys.version_info < (3, 8):
     from typing_extensions import Protocol
@@ -12,7 +12,7 @@ else:
 class Task(Protocol):
     def __call__(
         self,
-    ) -> Union[Awaitable[Iterable[Optional[Task]]], Iterable[Optional[Task]]]:
+    ) -> Union[Awaitable[Union[None, Iterable[Task]]], Union[None, Iterable[Task]]]:
         ...
 
 
